package com.hzya.frame.webapp.entrance.service.impl;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.extra.servlet.ServletUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.hzya.frame.web.action.ApplicationContextUtil;
import com.hzya.frame.web.cache.ServiceMethodCache;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import com.hzya.frame.webapp.entrance.service.IEntranceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * @Content统一根据请求服务处理相关业务service
 * @Author 👻👻👻👻👻👻👻👻👻👻 gjh
 * @Date 2022-12-07 8:47
 * @Version 1.0
 */
@Service(value = "entranceServiceImpl")
public class EntranceServiceImpl implements IEntranceService {

    private final Logger logger = LoggerFactory.getLogger(EntranceServiceImpl.class);
    @Autowired
    private ServiceMethodCache serviceMethodCache;


    @Override
    @DSTransactional()
    public JsonResultEntity doBusiness(ServletRequest servletRequest, ServletResponse servletResponse) throws Exception {
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        //tailuo 要访问的Bean名称
        String service = request.getHeader("tl");
        //bean方法
        String serviceMethod = request.getHeader("dj");
        String body = ServletUtil.getBody(servletRequest);
        //获取类
        Object object = null;
        try {
            object = ApplicationContextUtil.getBeanByName(service);
        } catch (SecurityException e) {
            return BaseResult.getFailureMessageEntity(service + "未找到");
        }
        //获取类下面的方法
        Method[] methods = object.getClass().getMethods();
        logger.info("methodsLength:" + methods.length);
        if (methods == null || methods.length == 0) {
            logger.info("当前methods:" + methods + "methodsLength = " + methods.length);
            return BaseResult.getFailureMessageEntity(service + "未找到" + serviceMethod + "方法");
        } else {
            logger.error("当前methods:" + methods + "methodsLength = " + methods.length);
        }
        for (Method m : methods) {
            logger.info("当前mnAME:" + m.getName() + "   :serviceMethod = " + serviceMethod.trim());
            if (null != m) {
                if (m.getName().equals(serviceMethod.trim())) {
                    if (!"doLogin".equals(m.getName())) {
                        try {
                            StpUtil.checkLogin();
                        } catch (NotLoginException e) {
                            logger.error("token无效");
                            return BaseResult.getFailureMessageEntity("token无效");
                        }
                    }
                    logger.info("m.getNameEquals成功");
                    JSONObject jsonObject = new JSONObject();
                    if (!ObjectUtils.isEmpty(body)) {
                        jsonObject.put("jsonStr", body);
                    }
                    JsonResultEntity jsonResultEntity;
                    try {
                        logger.info("invoke开始>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                        logger.info("请求的参数：jsonObject：" + jsonObject.toJSONString());
                        jsonResultEntity = (JsonResultEntity) m.invoke(object, jsonObject);
                        logger.info("invoke结束>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                        return jsonResultEntity;
                    } catch (Exception e) {
                        logger.error("METHOD doBusiness invokeException{}", e.getMessage());
                        if (e instanceof InvocationTargetException) {
                            Throwable targetException = ((InvocationTargetException) e).getTargetException();
                            if (targetException instanceof BaseSystemException) {
                                throw new BaseSystemException(targetException.getMessage());
                            } else {
                                logger.info("METHOD doBusiness :{}", e.getMessage());
                                throw e;
                            }
                        } else {
                            throw e;
                        }
                    }
                }
            }
        }
        return BaseResult.getFailureMessageEntity(service + "未找到" + serviceMethod + "方法");
    }
}
